@@ -222,7 +222,7 @@ tun_alloc(struct pmd_internals *pmd)
 	return fd;
 
 error:
-	if (fd > 0)
+	if (fd >= 0)
 		close(fd);
 	return -1;
 }
